<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Handler_Map extends Controller_Handler
{
    /** @var  Model_User_Teacher */
    protected $user;


    public function before() {
        parent::before();

        $this->user->checkAccess(User::RIGHTS_TEACHER);
    }


    private function privateDeleteSubmodule($SubmoduleID) {
        $result = Model_Map::deleteSubmodule($this->user->TeacherID, $SubmoduleID);
        return $result[0]['Num'];
    }

    public function action_ChangeDisciplineSubject() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('SubjectID', 'not_empty')
            ->rule('SubjectID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->ChangeDisciplineSubject($this->user->TeacherID, $_POST['SubjectID']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeStatusBonusModule() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('BonusRate', 'not_empty');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            if ($this->post['BonusRate'] == 'true') {
                $result = $map->AddModuleBonus($this->user->TeacherID);
                $data['action'] = 'add';
            } else {
                $result = $map->DeleteModuleBonus($this->user->TeacherID);
                $data['action'] = 'delete';
            }
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeDisciplineGrade() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('SubjectID', 'not_empty')
            ->rule('SubjectID', 'digit')
            ->rule('GradeID', 'not_empty')
            ->rule('GradeID', 'digit');

        if ($this->post->check()) {
            $result = Model_Discipline::load($_POST['DisciplineID'])
                ->changeGrade($this->user->TeacherID, $_POST['GradeID']);

            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeDisciplineControl() {
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('Control', 'not_empty');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->ChangeDisciplineControl($this->user->TeacherID, $_POST['Control']);
            $data['success'] = $result[0]['Num'];
        } else
            $data['success'] = 0;
        $this->response->body(json_encode($data));
    }

    public function action_ChangeDisciplineHours() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('Hours', 'not_empty')
            ->rule('Hours', 'digit')
            ->rule('Type', 'not_empty')
            ->rule('Type', 'digit');


        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->ChangeDisciplineHours($this->user->TeacherID, $_POST['Hours'], $_POST['Type']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_GetSubjectsList() {
        $data['success'] = false;
        $this->post->rule('FacultyID', 'not_empty')
            ->rule('FacultyID', 'digit');
        if ($this->post->check()) {
            $faculty = Model_Faculty::with($this->post['FacultyID']);
            $subjectsList = $faculty->getSubjects();
            $data['success'] = true;
            $this->response->body(json_encode($subjectsList));
        }
    }

    public function action_AddModule() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->addModule($this->user->TeacherID);

            $data['moduleID'] = $result[0]['Num'];
            if ($data['moduleID'] > 0) {
                // Добавление мероприятия
                // Т.к. модуль должен содержать хотя бы
                // 1 мероприятие
                $result = $map->addSubmodule($this->user->TeacherID, $data['moduleID'], 'CurrentControl');

                $data['submoduleID'] = $result[0]['Num'];
                if ($data['submoduleID'] > 0)
                    $data['success'] = true;
            }

        }
        $this->response->body(json_encode($data));
    }

    public function action_AddSubmodule() {
        $data['success'] = false;
        $this->post->rule('submoduleID', 'not_empty')
            ->rule('submoduleID', 'digit');

        if ($this->post->check()) {
            $result = Model_Map::addSubmodule($this->user->TeacherID, $_POST['submoduleID'], 'CurrentControl');
            $data['submoduleID'] = $result[0]['Num'];
            if ($data['submoduleID'] > 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeModuleName() {
        $data['success'] = false;
        $this->post->rule('ModuleID', 'not_empty')
            ->rule('ModuleID', 'digit')
            ->rule('ModuleName', 'not_empty');
        if ($this->post->check()) {
            $result = Model_Map::changeModuleName($this->user->TeacherID, $_POST['ModuleID'], $_POST['ModuleName']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeSubmoduleName() {
        $data['success'] = false;
        $this->post->rule('SubmoduleID', 'not_empty')
            ->rule('SubmoduleID', 'digit')
            ->rule('SubmoduleName', 'not_empty');
        if ($this->post->check()) {
            $result = Model_Map::changeSubmoduleName($this->user->TeacherID, $_POST['SubmoduleID'], $_POST['SubmoduleName']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_SwapModuleOrder() {
        $data['success'] = false;
        $this->post->rule('ModuleID1', 'not_empty')
            ->rule('ModuleID1', 'digit')
            ->rule('ModuleID2', 'not_empty')
            ->rule('ModuleID2', 'digit');
        if ($this->post->check()) {
            $result = Model_Map::swapModuleOrder($this->user->TeacherID, $_POST['ModuleID1'], $_POST['ModuleID2']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_SwapSubmoduleOrder() {
        $data['success'] = false;
        $this->post->rule('SubmoduleID1', 'not_empty')
            ->rule('SubmoduleID1', 'digit')
            ->rule('SubmoduleID2', 'not_empty')
            ->rule('SubmoduleID2', 'digit');
        if ($this->post->check()) {
            $result = Model_Map::swapSubmoduleOrder($this->user->TeacherID, $_POST['SubmoduleID1'], $_POST['SubmoduleID2']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_DeleteModule() {
        $data['success'] = false;
        $this->post->rule('ModuleID', 'not_empty')
            ->rule('ModuleID', 'digit');
        if ($this->post->check()) {
            $result = Model_Map::deleteModule($this->user->TeacherID, $_POST['ModuleID']);
            $data['success'] = ($result[0]['Num'] == 0);
        }
        $this->response->body(json_encode($data));
    }

    public function action_DeleteSubmodule() {
        $data['success'] = false;
        $this->post->rule('SubmoduleID', 'not_empty')
            ->rule('SubmoduleID', 'digit');
        if ($this->post->check()) {
            $result = $this->privateDeleteSubmodule($this->post['SubmoduleID']);
            $data['success'] = ($result == 0);
        }
        $this->response->body(json_encode($data));
    }

    public function action_ChangeSubmoduleMaxAndControl() {
        $data['success'] = false;
        $this->post->rule('SubmoduleID', 'not_empty')
            ->rule('SubmoduleID', 'digit')
            ->rule('MaxRate', 'not_empty')
            ->rule('ControlType', 'not_empty');
        if ($this->post->check()) {
            $result = Model_Map::changeSubmoduleMaxAndControl(
                $this->user->TeacherID, $_POST['SubmoduleID'], $_POST['MaxRate'], $_POST['ControlType']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_BindGroup() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('GroupID', 'not_empty')
            ->rule('GroupID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->bindGroup($this->user->TeacherID, $_POST['GroupID']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
            $data['code'] = $result[0]['Num'];
        }
        $this->response->body(json_encode($data));
    }

    public function action_UnbindGroup() {
        $data['success'] = false;
        $this->post->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit')
            ->rule('GroupID', 'not_empty')
            ->rule('GroupID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->unbindGroup($this->user->TeacherID, $_POST['GroupID']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_BindStudent() {
        $data['success'] = false;
        $this->post->rule('StudentID', 'not_empty')
            ->rule('StudentID', 'digit')
            ->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->BindStudent($this->user->TeacherID, $map['StudentID']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_UnbindStudent() {
        $data['success'] = false;
        $this->post->rule('StudentID', 'not_empty')
            ->rule('StudentID', 'digit')
            ->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit');

        $discipline = Model_Discipline::load($_POST['DisciplineID']);
        $map = Model_Map::of($discipline);

        if ($this->post->check()) {
            $result = $map->UnbindStudent($this->user->TeacherID, $_POST['StudentID']);
            if ($result[0]['Num'] == 0)
                $data['success'] = true;
        }
        $this->response->body(json_encode($data));
    }

    public function action_GetGroups() {
        $this->post->rule('GradeID', 'digit')
            ->rule('FacultyID', 'digit');
        if ($this->post->check()) {
            $grade = $this->post['GradeID'];
            $faculty = $this->post['FacultyID'];
            $groups = Model_Faculty::with($faculty)->getGroups($grade);
            $this->response->body(json_encode($groups));
        }
    }

    public function action_SearchStudents() {
        $searchResult = ['success' => 'false'];
        $this->post->rule('GradeID', 'digit')
            ->rule('GroupID', 'digit')
            ->rule('FacultyID', 'digit')
            ->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit');
        if ($this->post->check()) {
            $searchResult = Model_Students::search(
                $this->post['GradeID'],
                $this->post['GroupID'],
                $this->post['FacultyID'],
                $this->post['Name'],
                $this->post['DisciplineID']
            );
        }
        $this->response->body(json_encode($searchResult));
    }

    public function action_GetDepartments() {
        $this->post->rule('FacultyID', 'not_empty')
            ->rule('FacultyID', 'digit');
        if ($this->post->check()) {
            $faculty = Model_Faculty::with($this->post['FacultyID']);
            $departments = $faculty->getDepartments();
            $this->response->body(json_encode($departments));
        }
    }

    public function action_SearchTeachers() {
        $this->post->rule('DepartmentID', 'digit')
            ->rule('DisciplineID', 'not_empty')
            ->rule('DisciplineID', 'digit');

        if ($this->post->check()) {
            $teachers = Model_Teachers::search(
                $this->post['Last'],
                $this->post['FacultyID'],
                $this->post['DepartmentID'],
                $this->post['DisciplineID']
            );

            $this->response->body(json_encode($teachers));
        }
    }
}
